Exception: Funes::UnknownEvent
- Inherits:
-
StandardError
- Object
- StandardError
- Funes::UnknownEvent
- Defined in:
- lib/funes/unknown_event.rb
Overview
Raised when a projection encounters an event type it doesn’t know how to interpret.
By default, projections silently ignore unknown events. This error is only raised when a projection is configured with ‘raise_on_unknown_events`, which enforces strict event handling to catch missing interpretation blocks during development.